@@ -33,6 +33,9 @@
 #include "spawn.h"
 #include "notify.h"
 
+#undef MONERO_DEFAULT_LOG_CATEGORY
+#define MONERO_DEFAULT_LOG_CATEGORY "notify"
+
 namespace tools
 {
 
@@ -47,6 +50,8 @@ Notify::Notify(const char *spec)
 
   boost::split(args, spec, boost::is_any_of(" \t"), boost::token_compress_on);
   CHECK_AND_ASSERT_THROW_MES(args.size() > 0, "Failed to parse spec");
+  if (strchr(spec, '\'') || strchr(spec, '\"') || strchr(spec, '\\'))
+    MWARNING("A notification spec contains a quote or backslash: note that these are handled verbatim, which may not be the intent");
   filename = args[0];
   CHECK_AND_ASSERT_THROW_MES(epee::file_io_utils::is_file_exist(filename), "File not found: " << filename);
 }
@@ -57,7 +62,7 @@ static void replace(std::vector<std::string> &v, const char *tag, const char *s)
     boost::replace_all(str, tag, s);
 }
 
-int Notify::notify(const char *tag, const char *s, ...)
+int Notify::notify(const char *tag, const char *s, ...) const
 {
   std::vector<std::string> margs = args;
 
