@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2019, The Monero Project
+// Copyright (c) 2017-2020, The Monero Project
 // 
 // All rights reserved.
 // 
@@ -33,40 +33,13 @@
 #include "cryptonote_basic/cryptonote_format_utils.h"
 #include "fuzzer.h"
 
-class BulletproofFuzzer: public Fuzzer
-{
-public:
-  virtual int run(const std::string &filename);
+BEGIN_INIT_SIMPLE_FUZZER()
+END_INIT_SIMPLE_FUZZER()
 
-private:
-};
-
-int BulletproofFuzzer::run(const std::string &filename)
-{
-  std::string s;
-
-  if (!epee::file_io_utils::load_file_to_string(filename, s))
-  {
-    std::cout << "Error: failed to load file " << filename << std::endl;
-    return 1;
-  }
+BEGIN_SIMPLE_FUZZER()
   std::stringstream ss;
-  ss << s;
+  ss << std::string((const char*)buf, len);
   binary_archive<false> ba(ss);
   rct::Bulletproof proof = AUTO_VAL_INIT(proof);
-  bool r = ::serialization::serialize(ba, proof);
-  if(!r)
-  {
-    std::cout << "Error: failed to parse bulletproof from file  " << filename << std::endl;
-    return 1;
-  }
-  return 0;
-}
-
-int main(int argc, const char **argv)
-{
-  TRY_ENTRY();
-  BulletproofFuzzer fuzzer;
-  return run_fuzzer(argc, argv, fuzzer);
-  CATCH_ENTRY_L0("main", 1);
-}
+  ::serialization::serialize(ba, proof);
+END_SIMPLE_FUZZER()
