@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2019, The Monero Project
+// Copyright (c) 2017-2020, The Monero Project
 // 
 // All rights reserved.
 // 
@@ -33,36 +33,10 @@
 #include "cryptonote_basic/cryptonote_format_utils.h"
 #include "fuzzer.h"
 
-class TransactionFuzzer: public Fuzzer
-{
-public:
-  virtual int run(const std::string &filename);
+BEGIN_INIT_SIMPLE_FUZZER()
+END_INIT_SIMPLE_FUZZER()
 
-private:
-};
-
-int TransactionFuzzer::run(const std::string &filename)
-{
-  std::string s;
-
-  if (!epee::file_io_utils::load_file_to_string(filename, s))
-  {
-    std::cout << "Error: failed to load file " << filename << std::endl;
-    return 1;
-  }
+BEGIN_SIMPLE_FUZZER()
   cryptonote::transaction tx = AUTO_VAL_INIT(tx);
-  if(!parse_and_validate_tx_from_blob(s, tx))
-  {
-    std::cout << "Error: failed to parse transaction from file  " << filename << std::endl;
-    return 1;
-  }
-  return 0;
-}
-
-int main(int argc, const char **argv)
-{
-  TRY_ENTRY();
-  TransactionFuzzer fuzzer;
-  return run_fuzzer(argc, argv, fuzzer);
-  CATCH_ENTRY_L0("main", 1);
-}
+  parse_and_validate_tx_from_blob(std::string((const char*)buf, len), tx);
+END_SIMPLE_FUZZER()
