@@ -465,7 +465,7 @@ class Daemon(object):
 
     def in_peers(self, in_peers):
         in_peers = {
-            'client': client,
+            'in_peers': in_peers,
         }
         return self.rpc.send_request('/in_peers', in_peers)
 
@@ -554,6 +554,16 @@ class Daemon(object):
         }
         return self.rpc.send_json_rpc_request(flush_cache)
 
+    def sync_txpool(self):
+        sync_txpool = {
+            'method': 'sync_txpool',
+            'params': {
+            },
+            'jsonrpc': '2.0',
+            'id': '0'
+        }
+        return self.rpc.send_json_rpc_request(sync_txpool)
+
     def rpc_access_info(self, client):
         rpc_access_info = {
             'method': 'rpc_access_info',
